import logging

from jeelink import helper
from jeelink.device.pca301 import PCADevice
from jeelink.gateway import JeeLink, jeelink_register, jeelink_pattern

_LOGGER = logging.getLogger(__name__)


@jeelink_register
class PCAJeeLink(JeeLink):
    def __init__(self, device_class=PCADevice):
        """Initialize the pca device."""
        super().__init__(device_class)

    def send_command(self, device_id, channel_id, command, data):
        address = helper.serialize(device_id).replace(" ", ",")
        self._write(f"{channel_id},{command},{address},{data},255,255,255,255s")

    @jeelink_pattern(f"OK 24 (\\d+) 4((?: \\d+){3})((?: \\d+){5})")
    def _pca_data(self, data):
        self.set_available(True)
        self._device_data(*data[0])

    @jeelink_pattern(f"L 24 (\\d+) \\d : (\\d+) 4((?: \\d+){3})((?: \\d+){5})")
    def _requested_device(self, data):
        number, channel, address, data = data[0]
        self.force_polling(number)
        self._add_device(helper.deserialize(address))
        self._device_data(channel, address, data)

    def _device_data(self, channel, address, raw_data):
        if device := self._devices.get(helper.deserialize(address)):
            data = [int(value) for value in raw_data.split(" ") if value]
            device.set_state(int(data[0]))
            device.set_power((int(data[1]) * 256 + int(data[2])) / 10.0)
            device.set_consumption((int(data[3]) * 256 + int(data[4])) / 100.0)
            device.set_channel(int(channel))
        else:
            self._add_device(helper.deserialize(address))
            self._device_data(channel, address, raw_data)

    def request_devices(self):
        self._write("l")

    def request_version(self):
        self._write("v")

    def set_led_off(self):
        """ Turn on/off the blue LED """
        self._write("0a")

    def set_led_on(self):
        """ Turn on/off the blue LED """
        self._write("1a")

    def force_polling(self, number):
        self._write(f"{number}p")

    def set_available(self, available):
        super().set_available(available)
        if not self._devices:
            self._init_connection()

    def _init_connection(self):
        self.request_version()
        self.set_led_off()
        self.request_devices()



import asyncio
import logging

import serial_asyncio
from serial import SerialException
from serial.tools import list_ports

from jeelink.sketch.pca301 import PCAJeeLink

_LOGGER = logging.getLogger(__name__)


def available_ports():
    return [port[0] for port in sorted(list_ports.comports())]


async def setup(port, baud=57600, device_class=PCAJeeLink):
    loop = asyncio.get_event_loop()
    try:
        _, jeelink = await serial_asyncio.create_serial_connection(loop, device_class, port, baudrate=baud)
    except SerialException as e:
        _LOGGER.error(e)
        raise ConnectionError
    return jeelink


async def is_pca(port, baud=57600, device_class=PCAJeeLink, timeout=10):
    jeelink = await setup(port, baud, device_class)
    if jeelink:
        try:
            await jeelink.wait_available(timeout=timeout/2)
        except TimeoutError:
            return False
        waited = 0
        jeelink.request_version()
        while not jeelink.model and waited <= timeout/2:
            await asyncio.sleep(0.1)
            waited += 0.1
        if jeelink.model:
            return "pcaSerial" in jeelink.model
    return False


def serialize(device_id):
    return f"{int(device_id[:3])} {int(device_id[3:6])} {int(device_id[6:])}"


def deserialize(device_id):
    return "".join([f"{chunk:>03}" for chunk in device_id.split(" ") if chunk])



class LaCrosseDevice:
    def __init__(self, jeelink, device_id):
        self._sensor_id = device_id
        self._sensor_type = 0
        self._new_battery = None
        self._temperature = 0
        self._humidity = 0
        self._low_battery = None
        self._jeelink = jeelink

    @property
    def sensor_id(self):
        return self._sensor_id

    def set_sensor_id(self, sensor_id):
        self._sensor_id = sensor_id

    @property
    def sensor_type(self):
        return self._sensor_type

    def set_sensor_type(self, sensor_type):
        self._sensor_type = sensor_type

    @property
    def new_battery(self):
        return self._new_battery

    def set_new_battery(self, new_battery):
        self._new_battery = new_battery

    @property
    def temperature(self):
        return self._temperature

    def set_temperature(self, temperature):
        self._temperature = temperature

    @property
    def humidity(self):
        return self._humidity

    def set_humidity(self, humidity):
        self._humidity = humidity

    @property
    def low_battery(self):
        return self._low_battery

    def set_low_battery(self, low_battery):
        self._low_battery = low_battery



import asyncio
import logging
import time

_LOGGER = logging.getLogger(__name__)

UPDATE_INTERVAL_SECONDS = 30


class PCADevice:
    def __init__(self, jeelink, device_id):
        self._jeelink = jeelink
        self._id = device_id
        self._state = None
        self._power = None
        self._consumption = None
        self._channel = 0
        self._last_update = time.time()
        self._available = True
        self._name = f"PCA 301 - {self._id}"

    @property
    def name(self):
        return self._name

    @property
    def id(self):
        return self._id

    @property
    def available(self):
        return self._available

    def _available_check_running(self):
        name = f"{self._name} available check"
        if not any([name == task.get_name() for task in asyncio.all_tasks(asyncio.get_event_loop())]):
            asyncio.create_task(self._set_unavailable(), name=name)

    def set_available(self, available):
        if available:
            self._available_check_running()
        self._available = available

    @property
    def power(self):
        return self._power

    def set_power(self, power):
        self._last_update = time.time()
        self._power = power

    @property
    def consumption(self):
        return self._consumption

    def set_consumption(self, consumption):
        self._last_update = time.time()
        self._consumption = consumption

    @property
    def channel(self):
        return self._channel

    def set_channel(self, channel):
        self._last_update = time.time()
        self._channel = channel

    @property
    def state(self):
        return self._state

    def set_state(self, state):
        self._last_update = time.time()
        self._state = state

    @property
    def last_update(self):
        return self._last_update

    def turn_on(self):
        self._state = True
        self._jeelink.send_command(self._id, self._channel, command=5, data=1)

    def turn_off(self):
        self._state = False
        self._jeelink.send_command(self._id, self._channel, command=5, data=0)

    def status_request(self):
        self._jeelink.send_command(self._id, self._channel, command=4, data=0)

    def reset(self):
        self._jeelink.send_command(self._id, self._channel, command=4, data=1)

    def identify(self):
        self._jeelink.send_command(self._id, self._channel, command=6, data=0)

    async def _set_unavailable(self):
        while self._last_update > (time.time() - UPDATE_INTERVAL_SECONDS * 10):
            await asyncio.sleep(UPDATE_INTERVAL_SECONDS)
        self.set_available(False)
        logging.warning(f"Set {self.name} unavailable")

    def close(self):
        for task in asyncio.all_tasks(asyncio.get_event_loop()):
            if task.get_name() == f"{self._name} available check" and not task.cancelled():
                task.cancel()



import logging

from jeelink import JeeLink
from jeelink.device.lacrosse import LaCrosseDevice
from jeelink.gateway import jeelink_pattern, jeelink_register

_LOGGER = logging.getLogger(__name__)

"""
<n>a     set to 0 if the blue LED bothers
<n>c     use one of the possible data rates (for transmit on RFM #1)
<n>d     set to 1 to see debug messages
<n>f     initial frequency in kHz (5 kHz steps, 860480 ... 879515) 
<n>h     altituide above sea level
<n>m     bits 1: 17.241 kbps, 2 : 9.579 kbps, 4 : 8.842 kbps (for RFM #1)
<n>M     bits 1: 17.241 kbps, 2 : 9.579 kbps, 4 : 8.842 kbps (for RFM #2)
<n>o     set HF-parameter e.g. 50305o for RFM12 or 1,4o for RFM69
<n>p     transmitted the payload on the serial port 1: all, 2: only undecoded data
<n>r     use one of the possible data rates (for RFM #1)
<n>R     use one of the possible data rates (for RFM #2)
<id,..>s send the bytes to the address id
<n>t     0=no toggle, else interval in seconds (for RFM #1)
<n>T     0=no toggle, else interval in seconds (for RFM #2)
v        show version
x        test command 
<n>y     if 1 all received packets will be retransmitted  
<n>z     set to 1 to display analyzed frame data instead of the normal data
"""


@jeelink_register
class LaCrosseJeeLink(JeeLink):
    def __init__(self, device_class=LaCrosseDevice):
        super().__init__(device_class)

    @jeelink_pattern("OK 9 (\\d+) (\\d+) (\\d+) (\\d+) (\\d+)")
    def _temperature(self, raw_data):
        device_id, *data = [int(c) for c in raw_data[0]]
        if device := self._devices.get(device_id):
            device.set_sensor_type(data[0] & 0x7f)
            device.set_new_battery(True if data[0] & 0x80 else False)
            device.set_temperature(float(data[1] * 256 + data[2] - 1000) / 10)
            device.set_humidity(data[3] & 0x7f)
            device.set_low_battery(True if data[3] & 0x80 else False)
        else:
            self._add_device(device_id)
            self._temperature(raw_data)

    def set_led_off(self):
        """ Turn on the blue LED """
        self._write("0a")

    def set_led_on(self):
        """ Turn off the blue LED """
        self._write("1a")

    def request_version(self):
        self._write("v")



#!/usr/bin/env python3
from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="jeelink",
    version="0.2.0",
    author="Andre Basche",
    description="Python interface for jeelink sketches",
    long_description=long_description,
    license="MIT",
    project_urls={
        "GitHub": "https://github.com/Andre0512/lidl-plus",
        "PyPI": "https://pypi.org/project/lidl-plus/",
    },
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Console",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    platforms="any",
    py_modules=["jeelink"],
    packages=find_packages(),
    keywords="smart home automation",
    include_package_data=True,
    python_requires=">=3.8",
    install_requires=["pyserial", "pyserial-asyncio"],
)



import asyncio
import logging
import re
import sys

logging.basicConfig(stream=sys.stdout, level=logging.DEBUG,
                    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s")
_LOGGER = logging.getLogger(__name__)


def jeelink_register(cls):
    cls._pattern = {}
    for method_name in dir(cls):
        method = getattr(cls, method_name)
        if hasattr(method, '_prop'):
            cls._pattern.update({method_name: method._prop})
    return cls


def jeelink_pattern(*args):
    def wrapper(func):
        func._prop = args
        return func

    return wrapper


@jeelink_register
class JeeLink(asyncio.Protocol):
    def __init__(self, device_class):
        self._data = ""
        self._model = ""
        self._available = False
        self._writer = None
        self._device_class = device_class
        self._devices = {}
        self._handlers = []

    @property
    def available(self):
        return self._available

    @property
    def devices(self):
        return self._devices

    @property
    def model(self):
        return self._model

    @property
    def device_class(self):
        return self._device_class

    def set_device_class(self, device_class):
        self._device_class = device_class

    def set_available(self, available):
        self._available = available
        for device in self._devices.values():
            device.set_available(available)

    async def wait_available(self, timeout=10):
        waited = 0
        while not self.available and waited <= timeout:
            await asyncio.sleep(0.1)
            waited += 0.1
        if not self.available:
            raise TimeoutError

    def _process_data(self, read_data):
        for line in read_data.replace("\r", "").split("\n"):
            if line:
                _LOGGER.debug(f"Read  - {line}")
                for func, pattern in self._pattern.items():
                    if result := re.findall(pattern[0], line):
                        getattr(self, func)(result)

    def data_received(self, data: bytes) -> None:
        self._data = self._data + data.decode()
        if not self._data[-2:] == "\r\n":
            return
        try:
            self._process_data(self._data)
        finally:
            self._data = ""

    def connection_lost(self, exc):
        _LOGGER.error("Removed jeelink")
        self.set_available(False)

    def connection_made(self, transport):
        _LOGGER.info("Connected with jeelink")
        self._writer = transport
        self.set_available(True)

    def _write(self, text):
        _LOGGER.debug(f"Write - {text}")
        self._writer.write(text.encode())

    def _add_device(self, device_id):
        if device_id not in self._devices:
            self._devices[device_id] = self._device_class(self, device_id)

    @jeelink_pattern('\\[(.+?)]')
    def model_parser(self, result):
        self._model = result[0]
        if not self._available:
            self.set_available(True)
